define(['./module', 'moment'], function(module, moment) {

	module.controller('AssignCtrl', function($scope, $rootScope, $state, $stateParams, $ionicLoading, $timeout, $ionicPopup, $ionicHistory, Toast, Teachers, Resources, Tasks, AssignSucc, chivoxDatePicker) {

		var tea = Teachers.get();
		var bookdata = Resources.getChoosedBook();
		var grade = bookdata.grade;	// 年级及班级信息
		var selectedClasses = bookdata.selectedClasses;	// 选中了哪些班级
		var book = bookdata.book;
		var unitdata = Resources.getUnitSectionData();
		var scIdList = [];
		$scope.unitdata = unitdata;
		$scope.canGoToConfirm = true;

		// 提交后台的时间（开始时间）
		var startDate = moment().format('YYYY年MM月DD日');
		// 展示时间（开始时间）
		var startDateShow = moment().format('MM月DD日 dddd');
		$scope.daysDiff = 7;
		// 提交后端时间（截止时间）
		$scope.endDate = moment().add($scope.daysDiff, 'days').format('YYYY年MM月DD日');
		// 展示时间（截止时间）
		$scope.expiredDate = moment().add($scope.daysDiff, 'days').format('MM月DD日 dddd');

		// 从本地读取并拼接班级名称
		$scope.getClassesName = function() {
			var name = '';
			grade.scList.forEach(function(c) {
				if (selectedClasses[c.scId]) {
					name += c.scName;
					name += ' & ';
				};
			});
			if (name == '') {
				return name;
			};
			name = grade.cgyName + name.substr(0, name.length - 3);
			return name;
		}

		$scope.getSectionNames = function() {
			var name = '';
			unitdata.sectionIdList.forEach(function(s) {
				name += s.sectionName;
				name += ' ';
			})
			if (name) {
				name = name.substr(0, name.length - 1);
			};
			return name;
		}

		$scope.getTypeData = function() {
			var content = '';
			unitdata.taskTypeData.forEach(function(t) {
				content += t.taskName;
				content += ' (';
				t.types.forEach(function(type) {
					content += type.name;
					content += ' ';
				})
				content = content.substr(0, content.length - 1);
				content += ')';
				content += '<br>';
			})
			content = content.substr(0, content.length - 4);
			return content;
		}

		$scope.getStartTimeText = function() {
			var tempDate = moment().format('YYYY年MM月DD日');
			var startDateShowName = '';
			if (startDate == tempDate) {	// 表明是今天
				startDateShowName = '今天';
			} else {
				startDateShowName = startDateShow;
			};
			return startDateShowName;
		}

		$scope.setDaysFromNow = function(days) {
			if (!$scope.canGoToConfirm) {
				return;
			};
			$scope.daysDiff = days;
			$scope.endDate = moment(startDate, 'YYYY年MM月DD日').add(days, 'days').format('YYYY年MM月DD日');
			$scope.expiredDate = moment(startDate, 'YYYY年MM月DD日').add(days, 'days').format('MM月DD日 dddd');
		}

		$scope.confirm = function() {
			$scope.canGoToConfirm = false;
			grade.scList.forEach(function(c) {
				if (selectedClasses[c.scId]) {
					scIdList.push(c.scId);
				};
			});
			var params = makeupParams();
			Tasks.addOneSetTask(tea.id, scIdList, book.bookId, unitdata.unitId, unitdata.unitName, params, unitdata.tasksMin, startDate, $scope.endDate, tea.token).then(function(data) {
				$state.go('tab.tasks');
				$scope.canGoToConfirm = true;
			}, function(err) {
				$scope.canGoToConfirm = true;
			})
		}

		/**
		 * 日期时间选择（开始日期）
		 */
		$scope.startDateTimePicker = function() {
			if (!$scope.canGoToConfirm) {
				return;
			};
			chivoxDatePicker.show(convertDateForPlugin(startDate)).then(function(success) {
				if ($scope.daysDiff == 0) {
					$scope.daysDiff = 7;
				};

				// 传给后台
				startDate = success.year + '年' + ((success.month < 10) ? ('0' + success.month) : success.month) + '月' + ((success.day < 10) ? ('0' + success.day) : success.day) + '日';
				startDateShow = success.month + "月" + success.day + "日 " + success.week;

				$scope.setDaysFromNow($scope.daysDiff);
			});
		}

		/**
		* 日期时间选择（截止日期）
		*/
		$scope.dateTimePicker = function() {
			if (!$scope.canGoToConfirm) {
				return;
			};
			chivoxDatePicker.multipleShow(convertDateForPlugin(startDate), convertDateForPlugin($scope.endDate)).then(function(success) {
				$scope.daysDiff = 0; // 通过插件选时间，按钮置灰

				// 传给后台
				$scope.endDate = success.year + '年' + ((success.month < 10) ? ('0' + success.month) : success.month) + '月' + ((success.day < 10) ? ('0' + success.day) : success.day) + '日';

				$scope.expiredDate = success.month + "月" + success.day + "日 " + success.week;
			});
		}

		// 将日期转换为插件可以的形式
		function convertDateForPlugin(dateStr) {
			return moment(dateStr, 'YYYY年MM月DD日').format('YYYY-MM-DD');
		}

		function makeupParams() {
			var tempList = JSON.parse(JSON.stringify(unitdata.sectionIdList));
			var tempArray = [];
			unitdata.taskTypeData.forEach(function(tt) {
				tt.types.forEach(function(t) {
					t.contentRange = 1;
					t.item_list = "";
					tempArray.push(t);
				})
			})
			tempList.forEach(function(s) {
				s.taskTypeList = tempArray;
			})
			return tempList;
		}

		$scope.goBack = function() {
			$ionicHistory.goBack();
		}
	})
})