define(['./module'], function(module) {

	module.controller('BooksCtrl', function($scope, $rootScope, $state, $stateParams, $ionicLoading, $timeout, $ionicPopup, $ionicHistory, Teachers, Resources, Tasks, Toast, Config) {

		var tea = Teachers.get();
		var gradeId;
		var currGrade;
		var selectedClasses = {};
		var bookId;
		var currTextbook;
		var firstGetTextbookInfo = true;
		var condition = $stateParams.condition;

		$scope.grades = [];
		$scope.classList = [];
		$scope.textbookData = null;

		Resources.getSCInfoByTeaId(tea.id, tea.token).then(function(data) {
			$scope.grades = data;
			var bookdata = Resources.getChoosedBook();
			if (bookdata) {	// 用localStorage中的数据初始化年级和班级
				bookdata = JSON.parse(JSON.stringify(bookdata))
				gradeId = bookdata.grade.cgyId;
				currGrade = bookdata.grade;
				currGrade.selectNum = 0;
				$scope.classList = bookdata.grade.scList;
				// 选中特定班级
				selectedClasses = bookdata.selectedClasses;
				$scope.classList.forEach(function(c) {
					if (selectedClasses[c.scId]) {
						$timeout(function() {
							$scope.selectOneClass(c.scId, true);
							$('#' + 'class-' + c.scId).prop("checked", true);
						}, 200);
					};
				});
			} else {	// 用默认值初始化年级和班级
				if ($scope.grades && $scope.grades.length > 0) {
					gradeId = $scope.grades[0].cgyId;
					currGrade = $scope.grades[0];
					$scope.classList = $scope.grades[0].scList;
					// 全选班级
					$timeout(function() {
						$('#allclassesitem').prop("checked", true);
						$scope.selectAll(true);
					}, 200);
				};
			}
		}, function(err) {
			
		})

		/**
		 * 请求选择班级的公共教材
		 */
		function onChange() {
			$scope.textbookData = null;
			currTextbook = null;
			bookId = null;
			var cIds = [];
			$scope.classList.forEach(function(c) {
				if (selectedClasses[c.scId]) {
					cIds.push(c.scId);
				};
			});
			if (cIds.length <= 0) {
				return;
			};
			Tasks.getTeacherBookDetailByScId(tea.id, cIds, tea.token).then(function(data) {
				$scope.textbookData = data;
				if (firstGetTextbookInfo) {
					firstGetTextbookInfo = false;
					var bookdata = Resources.getChoosedBook();
					if (bookdata) {	// 用localStorage中的数据初始化教材
						bookdata = JSON.parse(JSON.stringify(bookdata))
						bookId = bookdata.book.bookId;
						currTextbook = bookdata.book;
					} else {	// 用默认值初始化教材
						if (data.bookList && data.bookList.length > 0) {
							bookId = data.bookList[0].bookId;
							currTextbook = data.bookList[0];
						};
					}
				};
			}, function(err) {

			});
		}

		/**
		 * 选择年级
		 */
		$scope.selectGrade = function(id, grade) {
			if (id != gradeId) {	// 点击的不是同一个年级
				// 班级全不选
				$timeout(function() {
					$('#allclassesitem').prop("checked", false);
					$scope.selectAll(false);
				}, 200);
				selectedClasses = {};
				// 教材数据清空
				$scope.textbookData = null;
				currTextbook = null;
			}
			gradeId = id;
			currGrade = grade;
			for (var i = 0; i < $scope.grades.length; i++) {
				var grade = $scope.grades[i];
				if (grade.cgyId == id) {
					$scope.classList = grade.scList;
					break;
				};
			};
		}

		$scope.selectTextbook = function(id, book) {
			bookId = id;
			currTextbook = book;
		}

		$scope.isGradeSelected = function(id) {
			if (id == gradeId) {
				return true;
			} else {
				return false;
			};
		}

		$scope.isTextbookSelected = function(id) {
			if (id == bookId) {
				return true;
			} else {
				return false;
			};
		}

		/**
		 * 教材是否可用
		 */
		$scope.isBookValid = function(book) {
			if (book.isValid) {
				return true;
			};
			return false;
		}

		/**
		 * 选中所有班级或全不选中班级
		 */
		$scope.selectAll = function(value) {
			if (!$scope.classList || $scope.classList.length == 0) {
				return;
			};
			$scope.classList.forEach(function(c) {
				selectedClasses[c.scId] = value;
				$('#' + 'class-' + c.scId).prop("checked", typeof value === 'undefined' ? false : value);
			});
			if (value) {
				currGrade.selectNum = currGrade.scList.length;
			} else {
				currGrade.selectNum = 0;
			}
			onChange();
		}

		/**
		 * 选择单个班级
		 */
		$scope.selectOneClass = function(id, value) {
			selectedClasses[id] = value;
			if (value) {
				if (!currGrade.selectNum) {
					currGrade.selectNum = 1;
				} else {
					currGrade.selectNum += 1;
				};
				if (currGrade.selectNum >= currGrade.scList.length) {
					$('#allclassesitem').prop("checked", true);
				}
			} else {
				currGrade.selectNum -= 1;
				if (currGrade.selectNum < currGrade.scList.length) {
					$('#allclassesitem').prop("checked", false);
				}
			}
			onChange();
		}

		$scope.goBack = function() {
			if (condition == 'nomal') {
				$ionicHistory.goBack();
			}else if (condition == 'unNomal') {
				$state.go('tab.tasks');
			};
			
		}

		$scope.getHeight = function() {
			if ($scope.showTextbookEmpty()) {
				return 3.76;
			};
			var realNum = 0;
			$scope.textbookData.bookList.forEach(function(book) {
				if (book.isValid) {
					realNum++;
				};
			})
			return 2.96 * Math.ceil(realNum / 3) + 0.92;
		}

		$scope.confirmTextbook = function() {
			var cIds = [];
			$scope.classList.forEach(function(c) {
				if (selectedClasses[c.scId]) {
					cIds.push(c.scId);
				};
			});
			if (cIds.length <= 0) {
				Toast.show(Config.unselectedClass);
				return;
			};
			if (!currTextbook) {
				Toast.show(Config.unselectedTextbook);
				return;
			};
			Resources.setChoosedBook({
				grade: currGrade,
				selectedClasses: selectedClasses,
				book: currTextbook
			});
			$state.go('tab.units');
		}

		$scope.canGotoNext = function() {
			if (currTextbook) {
				return true;
			};
			return false;
		}

		$scope.showTextbookEmpty = function() {
			if (!$scope.textbookData 
				|| !$scope.textbookData.bookList 
				|| $scope.textbookData.bookList.length <= 0) {
				return true;
			};
			var realNum = 0;
			$scope.textbookData.bookList.forEach(function(book) {
				if (book.isValid) {
					realNum++;
				};
			})
			if (realNum <= 0) {
				return true;
			};
			return false;
		}

		$scope.getGradeAreaHeight = function() {
			var rows = Math.ceil($scope.grades.length / 3);
			return 1.28 + rows * 0.62 + (rows - 1) * 0.34;
		}
	})
})