define(['./module'], function(module) {

	module.controller('UnitsCtrl', function($scope, $rootScope, $state, $stateParams, $ionicLoading, $timeout, $ionicPopup, $ionicHistory, $ionicScrollDelegate, Toast, Teachers, Resources, Tasks) {

		var tea = Teachers.get();
		var bookdata = Resources.getChoosedBook();
		var grade = bookdata.grade;	// 年级及班级信息
		var selectedClasses = bookdata.selectedClasses;	// 选中了哪些班级
		var book = bookdata.book;
		var unitIdForRadio = -1;	// 默认unit前的单选框都不选中
		var currUnitId;
		var currUnitName;
		var sectionIdList = [];

		$scope.allBookData = null;
		$scope.units = [];	// 教材的详细单元信息

		var scIdList = [];
		grade.scList.forEach(function(c) {
			if (selectedClasses[c.scId]) {
				scIdList.push(c.scId);
			};
		});

		// 获取选中教材的详细信息
		Tasks.getUnitInfoByBookId(tea.id, book.bookId, tea.token, scIdList).then(function(data) {
			$scope.allBookData = data;
			$scope.units = data.unitInfo;
			disposeRecent();
		}, function(err) {
			Resources.cleanChoosedBook();
			$state.go('tab.books', {
				condition: 'unNomal'
			});
		});

		// 从本地读取并拼接班级名称
		$scope.getClassesName = function() {
			var name = '';
			grade.scList.forEach(function(c) {
				if (selectedClasses[c.scId]) {
					name += c.scName;
					name += ' & ';
				};
			});
			if (name == '') {
				return name;
			};
			name = grade.cgyName + name.substr(0, name.length - 3);
			return name;
		}

		$scope.changeBook = function() {
			$state.go('tab.books', {
				condition: 'nomal'
			});
		}

		$scope.goBack = function() {
			$state.go('tab.tasks');
		}

		/**
		 * 全选或全不选
		 */
		$scope.selectAll = function(event, unitId, unit) {
			currUnitId = unitId;
			currUnitName = unit.unitName;
			if (unitId == unitIdForRadio) {	// 已经全选了
				unitIdForRadio = -1;
				unit.sectionList.forEach(function(section) {
					section.checked = false;
					$("#section-" + unitId + section.sectionId).prop("checked",  false);
				});
				unit.selectNum = 0;
			} else {	// 还未全选
				unitIdForRadio = unitId;
				// 其他unit下的选择全部清空
				clearOtherUnit(unitId);

				unit.sectionList.forEach(function(section) {
					section.checked = true;
					$("#section-" + unitId + section.sectionId).prop("checked",  true);
				});
				unit.selectNum = unit.sectionList.length;
			};

			$scope.stopEvent(event);
			onChange();
		}

		/**
		 * 选中或取消选中单个题型
		 */
		$scope.selectOnly = function(unit, section, value) {
			currUnitId = unit.unitId;
			currUnitName = unit.unitName;

			// 其他unit下的选择全部清空
			clearOtherUnit(unit.unitId);

			section.checked = value;
			if (value) {
				if (!unit.selectNum) {
					unit.selectNum = 1;
				} else {
					unit.selectNum += 1;
				};
				if (unit.selectNum >= unit.sectionList.length) {	// 选中该单元的radio
					unitIdForRadio = unit.unitId;
				}
			}else{
				unit.selectNum -= 1;
				if (unit.selectNum < unit.sectionList.length) {	// 取消选中该单元的radio
					unitIdForRadio = -1;
				}
			}
			onChange();
		}

		/**
		 * 获取该子单元是否被选中
		 */
		$scope.getSectionSelection = function(section) {
			return section.checked;
		}

		$scope.isUnitAllSelected = function(unitId) {
			if (unitId == unitIdForRadio) {
				return true;
			} else {
				return false;
			};
		}

		/**
		 * 阻止事件传递
		 */
		$scope.stopEvent = function(event) {
			event.stopPropagation();
		}

		$scope.toggle = function(unit) {
			currUnitId = unit.unitId;
			currUnitName = unit.unitName;
			if ($scope.isShown(unit)) {
				$scope.shownUnit = null;
			} else {
				$scope.shownUnit = unit;
			}
			$ionicScrollDelegate.resize();
		}

		$scope.isShown = function(unit) {
			return $scope.shownUnit === unit;
		}

		$scope.isUnitSelected = function(unitId) {
			if (unitId == currUnitId) {
				return true;
			} else {
				return false;
			};
		}

		/**
		 * 是否显示该unit（如果该unit下没有section，则不显示unit）
		 */
		$scope.isShowItem = function(unit) {
			if (!unit.sectionList) {
				return false;
			};
			if (unit.sectionList.length > 0) {
				return true;
			} else {
				return false;
			};
		}

		$scope.canGoNext = function() {
			if (sectionIdList.length == 0) {
				return false;
			} else {
				return true;
			};
		}

		/**
		 * 下一步
		 */
		$scope.next = function() {
			if ($scope.canGoNext()) {
				Resources.setUnitSectionData({
					unitId: currUnitId,
					unitName: currUnitName,
					imgUrl: $scope.allBookData.imgUrl,
					sectionIdList: sectionIdList
				});
				$state.go('tab.chooseTopic');
			};
		}

		function onChange() {
			var sectionIds = [];
			for (var i = 0; i < $scope.units.length; i++) {
				var unit = $scope.units[i];
				if (unit.unitId != currUnitId) {
					continue;
				} else {
					unit.sectionList.forEach(function(section) {
						if (section.checked) {
							sectionIds.push({
								sectionId: section.sectionId,
								sectionName: section.sectionName,
								sectionIndex: section.sectionIndex
							});
						};
					});
				};
			}
			sectionIdList = sectionIds;
		}

		/**
		 * 清空其他unit下的选中
		 */
		function clearOtherUnit(unitId) {
			if (unitIdForRadio != -1 && unitIdForRadio != unitId) {
				unitIdForRadio = -1;
			};
			$scope.units.forEach(function(unit) {
				if (unit.unitId != unitId) {
					unit.selectNum = 0;
					unit.sectionList.forEach(function(section) {
						section.checked = false;
						$("#section-" + unit.unitId + section.sectionId).prop("checked",  false);
					});
				};
			});
		}

		function disposeRecent() {
			var unit = null;
			var pastNum = -1;
			for (var i = 0; i < $scope.units.length; i++) {
				unit = $scope.units[i];
				if (unit.isUsed) {
					pastNum = i;
					break;
				};
			};
			if (pastNum >= 0 && unit) {
				$scope.toggle(unit);
				unit.sectionList.forEach(function(section) {
					if (section.isUsed) {
						$timeout(function() {
							$("#section-" + unit.unitId + section.sectionId).prop("checked",  true);
						}, 200)
						$scope.selectOnly(unit, section, true);
						$ionicScrollDelegate.scrollBy(0, getScollDistance(pastNum), false);
					};
				})
			};
		}

		function getScollDistance(pastNum) {
			var distance = (1.28 + pastNum * 1.2) * (window.innerWidth / 7.5);
			return distance;
		}
	})
})