define(['./module'], function(module) {

	module.directive('draggable', [function() {
		return {
			restrict: 'A',
			link: function(scope, elem, attrs) {
				new DraggableView({
					el: elem[0]
				})
			}
		}
	}])

	var DraggableView = ionic.views.View.inherit({
		/**
		 * Initialize a card with the given options.
		 */
		initialize: function(opts) {
			opts = ionic.extend({
			}, opts);

			ionic.extend(this, opts);

			this.el = opts.el;

			this.startX = this.startY = this.x = this.y = 0;

			this.bindEvents();
		},

		/**
		 * Bind drag events on the card.
		 */
		bindEvents: function() {
			var self = this;
			var el = this.el;
			ionic.onGesture('dragstart', function(e) {
				ionic.requestAnimationFrame(function() { self._doDragStart(e) });
			}, el);

			ionic.onGesture('drag', function(e) {
				ionic.requestAnimationFrame(function() { self._doDrag(e) });
			}, el);

			ionic.onGesture('dragend', function(e) {
				ionic.requestAnimationFrame(function() { self._doDragEnd(e) });
			}, el);
		},

		_doDragStart: function(e) {
			this.startX = this.x;
			this.startY = this.y;
		},

		_doDrag: function(e) {
			this.x = this.startX + e.gesture.deltaX;
			this.y = this.startY + e.gesture.deltaY;

			this.el.style[ionic.CSS.TRANSFORM] = 'translate3d(' + this.x + 'px, ' + this.y	+ 'px, 0)';
		},
		_doDragEnd: function(e) {
			
		}
	});
})