define([
	'app'
], function(app) {
	return app.config(function($stateProvider, $urlRouterProvider) {

		// Ionic uses AngularUI Router which uses the concept of states
		// Learn more here: https://github.com/angular-ui/ui-router
		// Set up the various states which the app can be in.
		// Each state's controller can be found in controllers.js
		$stateProvider

		.state('guide', {
			url: '/guide',
			templateUrl: 'tmpl/guide/index.html'
		})

		.state('login', {
			url: '/login',
			templateUrl: 'tmpl/login/login.html'
		})

		// setup an abstract state for the tabs directive
		.state('tab', {
			url: "/tab",
			abstract: true,
			templateUrl: "tmpl/layout.html",
			controller: 'TabCtrl'
		})

		// Each tab has its own nav history stack:

		.state('tab.tasks', {
			url: '/tasks',
			views: {
				'tab-tasks': {
					templateUrl: 'tmpl/tasks/index.html',
					controller: 'TasksCtrl'
				}
			}
		})

		// 作业概览页（属于作业tab）
		.state('tab.task', {
			url: '/task/:worksetId/:scId/:className',
			hideTabs: true,
			views: {
				'tab-tasks': {
					templateUrl: 'tmpl/tasks/task.html',
				}
			}
		})

		// 班级成绩页（属于作业tab）
		.state('tab.grade', {
			url: '/grade/:worksetId/:scId',
			hideTabs: true,
			views: {
				'tab-tasks': {
					templateUrl: 'tmpl/tasks/grade.html',
					controller: 'GradeCtrl'
				}
			}
		})

		// 学生成绩详情页（属于作业tab）
		.state('tab.student', {
			url: '/student/:wsId/:stuId',
			hideTabs: true,
			views: {
				'tab-tasks': {
					templateUrl: 'tmpl/tasks/student.html',
					controller: 'StudentCtrl'
				}
			}
		})

		.state('tab.books', {
			url: '/books/:condition',
			hideTabs: true,
			views: {
				'tab-tasks': {
					templateUrl: 'tmpl/tasks/books.html',
					controller: 'BooksCtrl'
				}
			}
		})

		.state('tab.units', {
			url: '/units',
			hideTabs: true,
			views: {
				'tab-tasks': {
					templateUrl: 'tmpl/tasks/units.html',
					controller: 'UnitsCtrl'
				}
			}
		})

		.state('tab.chooseTopic', {
			url: '/chooseTopic',
			hideTabs: true,
			views: {
				'tab-tasks': {
					templateUrl: 'tmpl/tasks/chooseTopic.html',
					controller: 'ChooseTopicCtrl'
				}
			}
		})

		.state('tab.assign', {
			url: '/assign',
			hideTabs: true,
			views: {
				'tab-tasks': {
					templateUrl: 'tmpl/tasks/assign.html',
					controller: 'AssignCtrl'
				}
			}
		})
		
		.state('tab.notices', {
			url: '/notices',
			views: {
				'tab-notices': {
					templateUrl: 'tmpl/notices/index.html',
					controller: 'NoticesCtrl'
				}
			}
		})

		// 公告页
		.state('tab.notice', {
			url: '/notice/:id/:jpush/:isRead',
			hideTabs: true,
			views: {
				'tab-notices': {
					templateUrl: function() {
						return 'tmpl/notices/notice' + '.html';
					}
				}
			}
		})

		// 作业概览页（属于消息tab）
		.state('tab.overview', {
			url: '/overview/:worksetId/:scId/:className',
			hideTabs: true,
			views: {
				'tab-notices': {
					templateUrl: 'tmpl/tasks/task.html'
				}
			}
		})

		// 班级成绩页（属于消息tab）
		.state('tab.overview-grade', {
			url: '/overview-grade/:worksetId/:scId',
			hideTabs: true,
			views: {
				'tab-notices': {
					templateUrl: 'tmpl/tasks/grade.html',
					controller: 'GradeCtrl'
				}
			}
		})

		// 学生成绩详情页（属于消息tab）
		.state('tab.overview-student', {
			url: '/overview-student/:wsId/:stuId',
			hideTabs: true,
			views: {
				'tab-notices': {
					templateUrl: 'tmpl/tasks/student.html',
					controller: 'StudentCtrl'
				}
			}
		})

		.state('tab.settings', {
			url: '/settings',
			views: {
				'tab-settings': {
					templateUrl: 'tmpl/settings/index.html',
					controller: 'SettingsCtrl'
				}
			}
		})

		.state('tab.feedback', {
			url: '/feedback/:version',
			hideTabs: true,
			views: {
				'tab-settings': {
					templateUrl: 'tmpl/settings/feedback.html',
					controller: 'FeedbackCtrl'
				}
			}
		});

		// if none of the above states are matched, use this as the fallback
		if (JSON.parse(localStorage.getItem('ngStorage-teacher'))) $urlRouterProvider.otherwise('/tab/tasks');
		else if (localStorage.getItem('ngStorage-hasShownGuided')) $urlRouterProvider.otherwise('/login');
		else $urlRouterProvider.otherwise('/guide');

	})
})