define(['./module'], function(module) {

	module.factory('PlatformUtil', function($rootScope, $ionicPlatform, $state, $ionicHistory, $cordovaNetwork, $cordovaStatusbar, $ionicPopup, $timeout, Config) {
		var PLATFORM_INITED = false;

		var backButtonAction = null;
		var keyboardAction = null;
		var pauseAction = null;

		return {
			init: function() {
				this.initRem();
				PLATFORM_INITED = true;
				window.cordova && cordova.plugins.Keyboard && cordova.plugins.Keyboard.disableScroll(true);

				var self = this;
				$ionicPlatform.registerBackButtonAction(function (e) {
					e.preventDefault();

					if (backButtonAction) { // 注册了回调
						if (backButtonAction(e) !== false) { // 回调没有返回false
							self._backButtonAction();
						}
					} else {
						self._backButtonAction();
					}
				}, 101)

				$ionicPlatform.on('pause', function(e) {
					self.triggerPauseAction(e);
				});

				$ionicPlatform.on('resume', function(e) {
					self.initRem();
					self.hideKeyboard();
					window.plugins.jPushPlugin.setApplicationIconBadgeNumber(0);
				});

				window.addEventListener("orientationchange", function(e) {
					document.body.parentNode.style.fontSize = window.innerWidth / 7.5 + 'px';
				}, false);

				window.addEventListener('native.keyboardshow', function(e) {
					keyboardAction && keyboardAction.show && keyboardAction.show(e);
				}, false);

				window.addEventListener('native.keyboardhide', function(e) {
					keyboardAction && keyboardAction.hide && keyboardAction.hide(e);
				}, false);

				$rootScope.$on('$cordovaNetwork:online', function(e, networkState) {
					// 默认wifi下直接下载学习资源，非wifi下首次下载需提示
					if (networkState == 'wifi') {
						Config.noWiFiDownload = false;
					}
				});

				$rootScope.$on('$stateChangeStart', function() {
					self.unregisterBackButtonAction();
					self.unregisterKeyboardAction();
				});
			},
			initRem: function() {
				document.body.parentNode.style.fontSize = window.innerWidth / 7.5 + 'px';
				
				$timeout(function() {
					document.body.parentNode.style.fontSize = window.innerWidth / 7.5 + 'px';
				}, 1000)
			},
			showStatusBar: function() {
				if (PLATFORM_INITED && !$cordovaStatusbar.isVisible()) {
					$cordovaStatusbar.show();
				}
				return this;
			},
			hideStatusBar: function() {
				PLATFORM_INITED && $cordovaStatusbar.hide();
				return this;
			},
			showKeyboard: function() {
				!cordova.plugins.Keyboard.isVisible && cordova.plugins.Keyboard.show();
			},
			hideKeyboard: function() {
				cordova.plugins.Keyboard.close();
				// android trick
				$timeout(function() {
					cordova.plugins.Keyboard.isVisible && cordova.plugins.Keyboard.close();
				}, 500)
			},
			registerPauseAction: function(action) {
				pauseAction = action;
				return this;
			},
			unregisterPauseAction: function() {
				pauseAction = null;
				return this;
			},
			triggerPauseAction: function(e) {
				pauseAction && pauseAction(e);
			},
			registerKeyboardAction: function(action) {
				keyboardAction = action;
				return this;
			},
			unregisterKeyboardAction: function() {
				keyboardAction = null;
				return this;
			},
			registerBackButtonAction: function(action) {
				backButtonAction = action;
				return this;
			},
			unregisterBackButtonAction: function() {
				backButtonAction = null;
				return this;
			},
			_backButtonAction: function() {
				if ($state.current.name === 'tab.tasks' || $state.current.name === 'login') { // 确认退出
					this._confirmExit();
				} else if ($ionicHistory.backView()) {
					$ionicHistory.goBack();
				} else {
					this._confirmExit();
				}
			},
			_confirmExit: function() {
				$ionicPopup.confirm({
					title: '提示',
					template: '确认退出应用？',
					okText: '确定',
					cancelText: '取消'
				}).then(function(state) {
					if (state) {
						ionic.Platform.exitApp();
					}
				})
			}
		}
	})
})