define(['./module'], function(module) {

	module.factory('Resources', function($q, $http, $timeout, $cordovaNetwork, $ionicPopup, $ionicLoading, $localStorage, Config, chivoxFileTransfer) {
		// 1：单词练习；2：句子；3：段落；4：对话；5：单词听说；6：单词背诵
		var types = ['word', 'sent', 'para', 'dialog', 'word'];

		return {
			taskData: null,
			unitSectionData: null,	// 保存了选择的单元和子单元信息
			getOneLearnByIdAndPart: function(stuId, resId, partType, token) {
				var q = $q.defer();

				// network
				var type = $cordovaNetwork.getNetwork();
				if (type !== 'wifi') {
					if (type == 'none') {
						$ionicPopup.alert({
							title: '网络不给力',
							template: Config.nonetwork,
							okText: '好的'
						}).then(function() {
							q.reject('not wifi');
						})
					} else {
						if (Config.noWiFiDownload) {
							getData();
						} else {
							$ionicPopup.confirm({
								title: '非Wi-Fi环境',
								template: Config.noWifiLoadRes,
								okText: '确定',
								cancelText: '取消'
							}).then(function(state) {
								if (state) {
									Config.noWiFiDownload = true;
									getData();
								} else {
									q.reject('not wifi');
								}
							})
						}
					}
				} else {
					getData();
				}

				function getData() {
					$ionicLoading.show({
						template: '<ion-spinner></ion-spinner><br/>加载中...'
					})
					$http.post(Config.httpUrl + '/resources/getOneLearnByIdAndPart', {
						isGetDSData: true,
						stuId: stuId,
						resId: resId,
						partType: partType,
						token: token
					}).success(function(data) {
						console.log('------getOneLearnByIdAndPart', data)
						if (Config.checkStatus(data.status)) {
							var res = data.data;
							if (partType != 6) {
								var uri = res.Resource.version + '/' + res.Resource.type + '/' + res.Resource.unit_name + '/';
								var audios = [];
								JSON.parse(res.Resource[types[partType - 1]]).forEach(function(item) {
									if (partType == 4) {	// 是对话类型
										item.content.forEach(function(c) {
											audios.push({uri: res.Resource.res_path, name: c.audio});
										});
									} else {	// 其他类型
										audios.push({uri: res.Resource.res_path, name: item.audio})
									}
								})
								console.log('------downloadFiles', audios)
								// 批量下载资源
								chivoxFileTransfer.downloadFiles(audios).then(function(success) {
								}, function(err) {
									// TODO
									$ionicPopup.alert({template: 'load resource', okText: '好的'})
									q.reject(err)
								}, function(progress) {
									if (progress == -1) { // progress为-1证明已经全部下载完毕
										$ionicLoading.hide();
										q.resolve(res);
									} else {
									}
								});
							} else {	// 单词背诵不需要下载资源
								$ionicLoading.hide();
								q.resolve(res);
							}
						} else {
							$ionicLoading.hide();
							$ionicPopup.alert({template: JSON.stringify(data), okText: '好的'})
						}
					}).error(function(err) {
						$ionicLoading.hide();
						q.reject(err);
					});
				}

				return q.promise;
			},
			getSCInfoByTeaId: function(teaId, token) {
				var q = $q.defer();
				var self = this;
				$http.post(Config.httpUrl + '/school_classs/getSCInfoByTeaId', {
					teacherId: teaId,
					token: token
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				})
				return q.promise;
			},
			getResBookAndContentByTea: function(teaId, version, type, token) {
				var q = $q.defer();
				var self = this;
				$http.post(Config.httpUrl + '/resources/getResBookAndContentByTea', {
					teacherId: teaId,
					version: version,
					type: type,
					token: token
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				})
				return q.promise;
			},
			setChoosedBook: function(bookdata) {
				$localStorage.choosedBook = bookdata;
			},
			cleanChoosedBook: function() {
				delete $localStorage.choosedBook;
			},
			getChoosedBook: function() {
				return $localStorage.choosedBook;
			},
			setChoosedUnit: function(unitdata) {
				this.unitdata = unitdata;
			},
			getChoosedUnit: function() {
				return this.unitdata;
			},
			getTaskData: function() {
				return this.taskData;
			},
			setTaskData: function(taskData) {
				this.taskData = taskData;
			},
			setUnitSectionData: function(data) {
				this.unitSectionData = data;
			},
			getUnitSectionData: function() {
				return this.unitSectionData;
			},
			setTaskTypeData: function(data) {
				if (this.unitSectionData) {
					this.unitSectionData.taskTypeData = data;
				};
			},
			setTasksMin: function(min) {
				if (this.unitSectionData) {
					this.unitSectionData.tasksMin = min;
				};
			},
			setTasksNum: function(num) {
				if (this.unitSectionData) {
					this.unitSectionData.tasksNum = num;
				};
			}
		}
	})
})