define(['./module', 'moment'], function(module, moment) {

	module.factory('Tasks', function($q, $http, $ionicPopup, Config) {

		return {
			taskCache: {},

			getTaskListBySCId: function(teacherId, scId, worksetStatus, page, row, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/TeacherTasks/getWorksetListByTeaId', {
					teacherId: teacherId,
					scId: scId,
					worksetStatus: worksetStatus,
					page: page,
					row: row,
					token: token
				}).success(function(data) {
					q.resolve(data.data.list);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			getTeacherBookDetailByScId: function(teacherId, scIdList, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/book_classes/getTeacherBookDetailByScId', {
					teacherId: teacherId,
					scIdList: scIdList,
					token: token
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			getSCReportBySCIdWSId: function(teacherId, scId, worksetId, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/works/getSCReportBySCIdWSId', {
					teacherId: teacherId,
					scId: scId,
					worksetId: worksetId,
					token: token
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			changeTaskExpireTime: function(teacherId, task_id, expire_time, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/tasks/changeTaskExpireTime', {
					teacherId: teacherId,
					task_id: task_id,
					expire_time: expire_time,
					token: token
				}).success(function(data) {
					q.resolve(data);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			claimWorksets: function(teacherId, worksetId, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/worksets/claimWorksets', {
					teacherId: teacherId,
					worksetId: worksetId,
					token: token
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			canceWorksets: function(teacherId, worksetId, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/worksets/canceWorksets', {
					teacherId: teacherId,
					worksetId: worksetId,
					token: token
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			setMarkRead: function(teacherId, school_class_id, task_id, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/tasks/setMarkRead', {
					teacherId: teacherId,
					school_class_id: school_class_id,
					task_id: task_id,
					token: token
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			getStuListReportByScIdWSId: function(teacherId, worksetId, scId, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/records/getStuListReportByScIdWSId', {
					teacherId: teacherId,
					worksetId: worksetId,
					scId: scId,
					token: token
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			getDetailReportByStuIdWSId: function(teacherId, wsId, stuId, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/records/getDetailReportByStuIdWSId', {
					teacherId: teacherId,
					wsId: wsId,
					studentId: stuId,
					token: token
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			addOneSetTask: function(teacherId, scIdList, bookId, unitId, unitName, params, estimateMin, startDate, expireDate, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/tasks/addOneSetTask', {
					teacherId: teacherId,
					scIdList: scIdList,
					bookId: bookId,
					unitId: unitId,
					unitName: unitName,
					params: params,
					estimateMin: estimateMin,
					startDate: startDate,
					expireDate: expireDate,
					token: token
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},


			getTaskByClassIdInStu: function(stuId, schoolClassId, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/tasks/getTaskByClassIdInStu', {
					isGetDSData: true,
					stuId: stuId,
					schoolClassId: schoolClassId,
					token: token
				}).success(function(data) {
					if (Config.checkStatus(data.status)) {
						var day = [];
						var week = [];
						var other = [];
						data.data.forEach(function(task) {
							var d, h, m;
							var taskDate = moment(task.Task.expire_date);
							var dateDiff = taskDate.diff(moment()) / 1000;
							if (dateDiff > 0) {
								d = Math.floor(dateDiff / 86400);
								h = Math.floor((dateDiff - d * 86400) / 3600);
								m = Math.floor((dateDiff - d * 86400 - h * 3600) / 60);
							} else { // 过期10分钟内还会出现在作业列表
								d = h = m = 0;
							}
							
							task.deadline = '';
							if (d) task.deadline += d + '天';
							task.deadline += h + '小时';
							task.deadline += m + '分';

							if (d < 1) day.push(task);
							else if (d >= 1 && d < 7) week.push(task);
							else other.push(task);
						});
						q.resolve({
							length: data.data.length,
							day: day,
							week: week,
							other: other
						});
					} else {
						$ionicPopup.alert({template: JSON.stringify(data), okText: '好的'});
					}
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			getExpiredTaskByClassIdInStu: function(stuId, schoolClassId, page, row, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/tasks/getExpiredTaskByClassIdInStu', {
					isGetDSData: true,
					stuId: stuId,
					schoolClassId: schoolClassId,
					page: page,
					row: row,
					token: token
				}).success(function(data) {
					if (Config.checkStatus(data.status)) {
						q.resolve(data.data.list);
					}
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			getDetailByUnitId: function(teacherId, unitId, sectionIdList, token) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/resources/getDetailByUnitId', {
					teacherId: teacherId,
					unitId: unitId,
					sectionIdList: sectionIdList,
					token: token
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			},
			getUnitInfoByBookId: function(teacherId, bookId, token, scIdList) {
				var q = $q.defer();

				$http.post(Config.httpUrl + '/school_classs/getUnitInfoByBookId', {
					teacherId: teacherId,
					bookId: bookId,
					token: token,
					scIdList: scIdList
				}).success(function(data) {
					q.resolve(data.data);
				}).error(function(err) {
					q.reject(err);
				});

				return q.promise;
			}
		}
	})
})