
(function(){

// 声明模块，以及它所依赖的模块，模块是可以嵌套的

  angular.module('ngCordovaChivoxPlugins', [
    'com.chivox.plugins'
  ]);

// 语音引擎插件
  angular.module('com.chivox.plugins.aiengine', ['ngCordova', 'com.chivox.plugins.string', 'com.chivox.plugins.chivoxFileTransfer'])

      .factory('AIEngine', ['$q', '$window', '$cordovaMedia', 'chivoxString', 'chivoxFileTransfer', function ($q, $window, $cordovaMedia, chivoxString, chivoxFileTransfer) {
        var media;
        var state = -2; // -2 未激活, -1 未初始化, 0 idle, 1 record, 2 playback
        var isPlaying = false; // 指示是否正在播放

        return {

          currTaskType: null,

          word: 'en.word.score',

          sent: 'en.sent.score',

          pred: 'en.pred.exam',

          memorize: 'en.pred.memorize',

          /**
           * 初始化引擎
           * @param userId
           * @returns {*}
           */
          init: function (userId) {
            var q = $q.defer();

            if (!$window.aiengine || state > -2) {
              q.resolve(null);
              return q.promise;
            }

            // cordova调用
            aiengine.init(function (result) {
              state = 0;
              q.resolve(result);
            }, function (err) {
              state = -2;
              q.reject(err);
            }, userId);

            return q.promise;
          },

          /**
           * 开始录音，录音有个默认录音时长（根据单词或句子，中文或英文，自动设置），不会一直录音下去的
           * 隐藏了引擎的启动与停止等操作，无需关注
           * @param refText 要评测的单词、句子、或中文等
           * @param coreType 内核
           * @returns {*}
           */
          startRecord: function (refText, coreType) {
            this.cancel();
            var q = $q.defer();

            if (!refText) {
              q.reject('ref text required');
              return q.promise;
            }

            if (coreType == '') {
              q.reject('no coreType');
              return q.promise;
            }

            // 得到单词数量
            var wordNum = refText.trim().split(/\W+/).length;
            if (coreType == this.word && wordNum > 1) {
              coreType = this.sent;
            }

            // 计算vad屏蔽时间
            var duration = 1;
            if (coreType !== this.word) {
              duration = 2 + 0.3 * wordNum;
            }

            var tmp = aiengine.volumeCallBack;

            aiengine.volumeCallBack = function(volume) {
              q.notify(volume);
            }

            state = 1;
            // cordova调用
            aiengine.start(function (result) {
              state = 0;
              console.log('TODO 验证录音时播放标准音时，是否会调用此回调，或者改成cancel时会不会回调')
              aiengine.volumeCallBack = tmp;
              q.resolve(result);
            }, function (err) {
              state = 0;
              aiengine.volumeCallBack = tmp;
              q.reject(err);
            }, refText, duration, coreType);

            return q.promise;
          },

          /**
           * 停止录音
           * @param success
           * @param error
           */
          stopRecord: function () {
            if (state == 1) {
              // cordova调用
              aiengine.stop(null, null);
              state = 0;
            }
          },

          /**
           * 播放录音
           * @returns {*}
           */
          playback: function (recordId) {
            this.cancel();
            var q = $q.defer();

            if (state < 0) {
              q.reject('uninit engine');
              return q.promise;
            }

            state = 2;
            // cordova调用
            aiengine.playback(function (result) {
              console.log('TODO 验证回放时播放标准音时，是否会调用此回调，或者改成cancel时会不会回调')
              state = 0;
              q.resolve(result);
            }, function (err) {
              state = 0;
              q.reject(err);
            }, recordId);

            return q.promise;
          },

          /**
           * 停止播放录音
           * @param success
           * @param error
           */
          stopPlayback: function () {
            if (state == 2) {
              // cordova调用
              aiengine.playbackStop(null, null);
              state = 0;
            }
          },

          /**
           * 播放原声音频
           * @param src 原声音频路径
           * @returns {*}
           */
          playAudio: function(resPath, audio) {
            var re = / /gi;
            var src = null;
            if (device.platform == 'iOS') {// ios audio name
              var cardAudioName = audio.replace(re, '_');
              src = chivoxFileTransfer.getAppRootDir() + chivoxFileTransfer.getLocalUri(chivoxFileTransfer.getLocalResPath(resPath)) + cardAudioName + '.mp3';
            } else {// android audio name
              src = chivoxFileTransfer.getAppRootDir() + chivoxFileTransfer.getLocalUri(chivoxFileTransfer.getLocalResPath(resPath)) + audio + '.mp3';
            }

            this.cancel();  // 重置引擎
            var q = $q.defer();

            var devicePlatform = device.platform;
            if (devicePlatform == 'iOS' && chivoxString.startWith(src, 'file://')) {
              src = src.substr(7);
            }
            media = $cordovaMedia.newMedia(src);
            media.then(function() {
              isPlaying = false;
              q.resolve();
            }, function() {
              isPlaying = false;
              q.reject();
            });
            media.play(); // Android
            isPlaying = true;
            return q.promise;
          },

          /**
           * 停止播放原声音频
           */
          stopAudio: function() {
            if (media && isPlaying) {
              media.stop();
              media.release();
              media = null;
              isPlaying = false;
            }
          },


          /**
           * 重置录音（停止录音并停止播放录音）
           * 重置引擎，取消了引擎当前的所有操作
           * @returns {*}
           */
          cancel: function () {
            // cordova调用
            // aiengine.cancel(null, null);
            switch (state) {
              case 1: // record
                this.stopRecord();
                break;
              case 2: // playback
                this.stopPlayback();
                break;
            }
            if (isPlaying) { // play
              this.stopAudio();
            }
          },

          /**
           * 销毁引擎实例，销毁所有已存在的引擎实例
           * @returns {*}
           */
          release: function () {
            // cordova调用
            if (state > -1) {
              aiengine.release(null, null);
            }
            this.stopAudio();
          },

          /**
           * 引擎是否初始化
           * @returns {*}
           */
          isInitialized: function () {
            return state >= 0;
          },

          /**
           * 是否正在录音
           * @returns {*}
           */
          isRecording: function () {
            return state == 1;
          },

          /**
           * 是否正在播放录音
           * @returns {*}
           */
          isReplaying: function () {
            return state == 2;
          },

          /**
           * 原声音频是否正在播放
           * @returns {boolean}
           */
          isAudioPlaying: function() {
            return isPlaying;
          }

        };

      }]);

  // 封装官方的FileTransfer插件，使更好用
  angular.module('com.chivox.plugins.chivoxFileTransfer', ['ngCordova', 'com.chivox.plugins.string'])

      .factory('chivoxFileTransfer', ['$q', '$cordovaFileTransfer', '$cordovaFile', 'chivoxString', function ($q, $cordovaFileTransfer, $cordovaFile, chivoxString) {

        var dirCount = 0;
        var p = 0;

        /**
         * 依次创建文件存放目录
         * @param tempRoot
         * @param dirs 要创建目录的数组
         */
        makeDirs = function(tempRoot, dirs) {
          if (dirCount < dirs.length - 1) {
            $cordovaFile.checkDir(tempRoot, dirs[dirCount])
                .then(function (success) {
                  tempRoot += dirs[dirCount] + '/';
                  dirCount++;
                  makeDirs(tempRoot, dirs);
                }, function (error) {
                  $cordovaFile.createDir(tempRoot, dirs[dirCount], false)
                      .then(function (success) {
                        tempRoot += dirs[dirCount] + '/';
                        dirCount++;
                        makeDirs(tempRoot, dirs);
                      }, function (error) {
                        tempRoot += dirs[dirCount] + '/';
                        dirCount++;
                        makeDirs(tempRoot, dirs);
                      });
                });

          }
        }

        return {

          /**
           * 下载单个文件
           * @param url 要下载文件的网络地址，路径中可以包含空格，该方法会自动将所有的空格转化为%20
           * @param path 文件存储路径，不包括文件名，要区分ios和android系统
           * @param fileName 文件名，如：ant.mp3
           */
          downloadSingleFile: function(url, path, fileName) {
            var targetPath = path + fileName;
            var trustHosts = true;
            var options = {};
            options.timeout = 60 * 1000;

            var q = $q.defer();
            $cordovaFileTransfer.download(url, targetPath, options, trustHosts)
                .then(function(result) {
                  q.resolve(result);
                }, function(err) {
                  q.reject(err);
                }, function (progress) {
                  q.notify(progress);
                });

            return q.promise;
          },

          /**
           * 依次下载多个文件
           * @param json
           */
          downloadFiles: function(audios) {
            var q = $q.defer();
            if (audios.length <= 0) { // 没有资源需要下载
              q.resolve();
              return q.promise;
            }

            // 开始进行下载
            p = 0;
            this.checkFileAndDownload(q, audios);

            return q.promise;

          },

          /**
           * 向服务器上传文件
           * @param url 服务器地址
           * @param stuId 学生id
           * @param token
           */
          uploadFile: function(url, stuId, token) {
            var devicePlatform = device.platform;
            var filePath;
            if (devicePlatform == 'Android') {
              filePath = cordova.file.externalDataDirectory + '/record/' + 'audioZip.zip';
            } else if (devicePlatform == 'iOS') {
              filePath = this.getAppRootDir() + 'record/audioZip.zip';
            } else {
              return;
            }
            var options = new FileUploadOptions();
            options.fileKey = "userfile";
            options.fileName = filePath.substr(filePath.lastIndexOf('/') + 1);
            options.mimeType = "application/octet-stream";
            options.httpMethod = "POST";
            options.timeout = 60 * 1000;
            var params = {};
            params.stuId = stuId;
            params.token = token;
            options.params = params;

            var q = $q.defer();
            $cordovaFileTransfer.upload(url, filePath, options)
                .then(function(result) {
                  q.resolve(result);
                }, function(err) {
                  q.reject(err);
                }, function (progress) {
                  q.notify(progress);
                });
            return q.promise;
          },

          /**
           * 删除音频存放目录，只在上传成功之后执行
           */
          deleteRecordDir: function() {
            var devicePlatform = device.platform;
            var tempPath;
            if (devicePlatform == 'Android') {
              tempPath = cordova.file.externalDataDirectory;
            } else if (devicePlatform == 'iOS') {
              tempPath = this.getAppRootDir();
            } else {
              return;
            }
            $cordovaFile.checkDir(tempPath, "record")
                .then(function (success) {
                  if (devicePlatform === 'iOS') {
                    $cordovaFile.removeRecursively(tempPath, "record")
                      .then(function (success) {
                      }, function (error) {
                      });
                    } else {
                      if (!chivoxString.endWith(tempPath, '/')) {
                        tempPath = tempPath + '/';
                      }
                      customzip.delFolder(null, null, tempPath + 'record');
                    }
                  
                }, function (error) {
                });
          },

          /**
           * 得到应用的根目录，以Android为例：/storage/sdcard0/chAudio/；以ios为例：cordova.file.documentsDirectory + 'chAudio/'
           * @returns {*}
           */
          getAppRootDir: function() {
            var devicePlatform = device.platform;
            var tempRoot;
            var tempTatgetPath;
            if (devicePlatform == 'Android') {
              tempRoot = cordova.file.externalRootDirectory;
              tempTatgetPath = tempRoot + 'chAudio/';
            } else if (devicePlatform == 'iOS') {
              tempRoot = cordova.file.documentsDirectory;
              tempTatgetPath = tempRoot + 'chAudio/';
            } else {
              return;
            }
            return tempTatgetPath;
          },

          /**
           * 检查文件是否存在，如果不存在才去下载，供内部使用
           * @param q
           * @param tempTatgetPath
           * @param audios
           */
          checkFileAndDownload: function(q, audios) {
            var resPath = this.getLocalResPath(audios[p].uri);
            var resName = audios[p].name;
            var uri = this.getLocalUri(resPath);  // uri为内容服务器上的绝对路径

            // 创建所需的目录
            var tempPath = 'chAudio/' + uri;
            var dirs = tempPath.split('/');
            var devicePlatform = device.platform;
            var tempRoot;
            var tempTatgetPath;
            if (devicePlatform == 'Android') {
              tempRoot = cordova.file.externalRootDirectory;
            } else if (devicePlatform == 'iOS') {
              tempRoot = cordova.file.documentsDirectory;
            } else {
              return;
            }
            tempTatgetPath = tempRoot + 'chAudio/';
            dirCount = 0;
            makeDirs(tempRoot, dirs);

            // 将所有的空格转化为_
            var thePath = tempTatgetPath;
            thePath = thePath + uri;

            if (p < audios.length) {
              var theDownloadUrl = resPath + resName + ".mp3";  // 下载路径
              var self = this;

              $cordovaFile.checkFile(thePath, resName + ".mp3")
                  .then(function (success) {  // 文件存在，无需下载
                    p++;
                    if (p >= audios.length) {
                      q.notify(-1);
                    } else {
                      q.notify(p / audios.length);
                      self.checkFileAndDownload(q, audios);
                    }
                  }, function (error) { // 文件不存在，需要下载
                    self.downloadSingleFile(theDownloadUrl, thePath, resName + ".mp3").then(function(success) {
                      console.log('succ============', p)
                      p++;
                      if (p >= audios.length) {
                        q.notify(-1);
                      } else {
                        q.notify(p / audios.length);
                        self.checkFileAndDownload(q, audios);
                      }
                    }, function(err) {
                      console.log('err-------------', p)
                      p++;
                      if (p >= audios.length) {
                        q.notify(-1);
                      } else {
                        q.notify(p / audios.length);
                        self.checkFileAndDownload(q, audios);
                      }
                    });
                  });
            }
          },

          /**
           * 压缩文件
           */
          zip: function() {
            var q = $q.defer();
            customzip.zip(function() {
              q.resolve();
            }, function(err) {
              q.reject(err);
            });
            return q.promise;
          },

          /**
           * 得到指定文件夹或文件的总大小
           * @param path 指定路径
           */
          getDirTotalSize: function(path) {
            var q = $q.defer();
            customzip.getDirTotalSize(function(totalSize) {
              q.resolve(totalSize);
            }, function(err) {
              q.reject(err);
            }, path);
            return q.promise;
          },

          /**
           * 清除缓存文件
           */
          clearCache: function() {
            var devicePlatform = device.platform;
            var tempRoot;
            if (devicePlatform == 'Android') {
              tempRoot = cordova.file.externalRootDirectory;
            } else if (devicePlatform == 'iOS') {
              tempRoot = cordova.file.documentsDirectory;
            } else {
              return;
            }
            var q = $q.defer();
            $cordovaFile.checkDir(tempRoot, "chAudio")  // 验证是否存在
                .then(function (success) {
                  $cordovaFile.removeRecursively(tempRoot, "chAudio")
                      .then(function (success) {
                        q.resolve();
                      }, function (error) {
                        q.reject('remove dir error');
                      });
                }, function (error) {
                  q.reject('dir do not exist');
                });
            return q.promise;
          },

          /**
           * 获取URI
           */
          getLocalUri: function(path) {
            var re = / /gi;
            return path.substring(7).replace(re, '_');
          },
          getLocalResPath: function(path) {
            if (!chivoxString.startWith(path, 'http')) {
              return path.replace(/\/?web\/assets\/data\/course/g, 'http://resm.chivoxapp.com/files/cs');
            }
            return path;
          }

        };

      }]);

  // 封装极光推送插件，使更好用
  angular.module('com.chivox.plugins.jpush', [])

      .factory('chivoxJPush', ['$q', '$window', '$rootScope', function ($q, $window, $rootScope) {

        function getMsg(e) {
          var data = null;
          var msg = {};

          if (e.aps) { // ios
            data = e.aps;
            msg.alert = data.alert;
            msg.extra = {
              jtype: e.jtype,
              jdata: e.jdata
            };
          } else {
            console.log('------------', e)
            if (e.type == 'jpush.receiveNotification') data = $window.plugins.jPushPlugin.receiveNotification;
            else data = $window.plugins.jPushPlugin.openNotification;
            msg.alert = data.extras['cn.jpush.android.ALERT'];
            msg.extra = data.extras['cn.jpush.android.EXTRA'];
          }

          return msg;
        }

        var cvJpush = {
          StateActive: 0,
          StateBackground: 1,

          //启动（初始化）极光推送，没有重启功能
          initPush: function(){
            $window.plugins.jPushPlugin.init();
            $window.plugins.jPushPlugin.setDebugMode(false);

            /**
             * 添加打开通知时的事件监听（打开通知时，调用该回调函数），到JPushPlugin.js中的openNotificationInAndroidCallback函数中查看参数data的用法
             */
            document.addEventListener("jpush.openNotification", function (e) {
              $rootScope.$emit('jpush.openNotification', getMsg(e));
            }, false);

            document.addEventListener("jpush.receiveNotification", function (e) {
              $rootScope.$emit('jpush.receiveNotification', getMsg(e));
            }, false);

            /**
             * 添加接收到消息时的事件监听（接收到消息时，回调该函数），到JPushPlugin.js中的receiveMessageInAndroidCallback函数中查看参数data的用法
             */
            document.addEventListener("jpush.receiveMessage", function (e) {
              $rootScope.$emit('jpush.receiveMessage', getMsg(e));
            }, false);
          },

          //停止极光推送
          stopPush: function(){
            $window.plugins.jPushPlugin.stopPush();
          },

          //重启极光推送，当调用stopPush停止极光推送之后，只能再调用resumePush来重启极光推送，再次调用init是无法启动极光推送的
          resumePush: function(){
            $window.plugins.jPushPlugin.resumePush();
          },

          //设置标签和别名，用于清空之前设置的标签与别名，执行之后相当于没有设置过标签与别名
          setTagsWithAlias: function(tags,alias){
            $window.plugins.jPushPlugin.setTagsWithAlias(tags,alias);
          },

          //设置标签，多个tag用“,”分隔
          setTags: function(tags){
            $window.plugins.jPushPlugin.setTags(tags);
          },

          //设置别名
          setAlias: function(alias){
            $window.plugins.jPushPlugin.setAlias(alias);
          }

        };

        return cvJpush;

      }]);

  angular.module('com.chivox.plugins.toast', [])

      .factory('Toast', ['$timeout', function ($timeout) {

        return {

          shown: false,
          timer: null,

          show: function(msg, pos) {
            if (!msg) {
              console.log('toast', 'msg is null');
              return;
            }
            
            if (!window.plugins) {
              console.log('Toast', msg);
              return;
            }
            
            if (this.shown) {
              this.hide();
              $timeout.cancel(this.timer);
            }

            this.shown = true;

            window.plugins.toast.show(msg, 'short', pos || 'center');

            var self = this;
            this.timer = $timeout(function() {
              self.shown = false;
            }, 2000)
          },
          hide: function() {
            window.plugins.toast.hide();
          }

        };

      }]);

  angular.module('com.chivox.plugins.datepicker', [])

      .factory('chivoxDatePicker', ['$q', '$window', function ($q, $window) {

        return {

          show: function(date) {
            var q = $q.defer();

            if (!$window.datePicker) {
              q.resolve(null);
              return q.promise;
            }

            datePicker.show(function(success) {
              q.resolve(success);
            }, null, date);

            return q.promise;
          },
          multipleShow: function(startDate, endDate) {
            var q = $q.defer();

            if (!$window.datePicker) {
              q.resolve(null);
              return q.promise;
            }

            datePicker.multipleShow(function(success) {
              q.resolve(success);
            }, null, startDate, endDate);

            return q.promise;
          }

        };

      }]);

  // 扩展String的功能
  angular.module('com.chivox.plugins.string', [])

      .factory('chivoxString', ['$q', function ($q) {

        return {

          /**
           * 判断字符串是否以指定的字符串结尾
           * @param ori 原始字符串
           * @param str 指定的字符串
           * @returns {boolean}
           */
          endWith: function(ori, str) {
            if (str == null || str == "" || ori.length == 0 || str.length > ori.length)
              return false;
            if (ori.substring(ori.length - str.length) == str)
              return true;
            else
              return false;
            return true;
          },

          /**
           * 判断字符串是否以指定的字符串开始
           * @param ori 原始字符串
           * @param str 指定的字符串
           * @returns {boolean}
           */
          startWith: function(ori, str) {
            if (str == null || str == "" || ori.length == 0 || str.length > ori.length)
              return false;
            if (ori.substr(0, str.length) == str)
              return true;
            else
              return false;
            return true;
          }

        };

      }]);

angular.module('com.chivox.plugins.actionSheet', [])
.factory('$cvActionSheet', [
  '$rootScope',
  '$compile',
  '$animate',
  '$timeout',
  '$ionicTemplateLoader',
  '$ionicPlatform',
  '$ionicBody',
  'IONIC_BACK_PRIORITY',
function($rootScope, $compile, $animate, $timeout, $ionicTemplateLoader, $ionicPlatform, $ionicBody, IONIC_BACK_PRIORITY) {

  var SHEET_TPL =
  '<div class="action-sheet-backdrop cv-action-sheet active" ng-class="cssClass" ng-click="cancel()">' +
    '<div class="action-sheet-wrapper">' +
      '<div class="action-sheet" ng-class="{\'action-sheet-has-icons\': $actionSheetHasIcon}">' +
        '<div class="action-sheet-group action-sheet-options">' +
          '<div class="action-sheet-title" ng-if="titleText" ng-bind-html="titleText"></div>' +
          '<button class="button action-sheet-option" ng-click="buttonClicked($index)" ng-repeat="b in buttons" ng-bind-html="b.text" ng-class="{selected: b.selected}"></button>' +
        '</div>' +
      '</div>' +
    '</div>' +
  '</div>';

  return {
    show: actionSheet
  };

  /**
   * @ngdoc method
   * @name $ionicActionSheet#show
   * @description
   * Load and return a new action sheet.
   *
   * A new isolated scope will be created for the
   * action sheet and the new element will be appended into the body.
   *
   * @param {object} options The options for this ActionSheet. Properties:
   *
   *  - `[Object]` `buttons` Which buttons to show.  Each button is an object with a `text` field.
   *  - `{string}` `titleText` The title to show on the action sheet.
   *  - `{string=}` `cancelText` the text for a 'cancel' button on the action sheet.
   *  - `{string=}` `destructiveText` The text for a 'danger' on the action sheet.
   *  - `{function=}` `cancel` Called if the cancel button is pressed, the backdrop is tapped or
   *     the hardware back button is pressed.
   *  - `{function=}` `buttonClicked` Called when one of the non-destructive buttons is clicked,
   *     with the index of the button that was clicked and the button object. Return true to close
   *     the action sheet, or false to keep it opened.
   *  - `{function=}` `destructiveButtonClicked` Called when the destructive button is clicked.
   *     Return true to close the action sheet, or false to keep it opened.
   *  -  `{boolean=}` `cancelOnStateChange` Whether to cancel the actionSheet when navigating
   *     to a new state.  Default true.
   *  - `{string}` `cssClass` The custom CSS class name.
   *
   * @returns {function} `hideSheet` A function which, when called, hides & cancels the action sheet.
   */
  


  function noop() {}

  function actionSheet(opts) {
    var scope = $rootScope.$new(true);

    $.extend(scope, {
      cancel: noop,
      destructiveButtonClicked: noop,
      buttonClicked: noop,
      $deregisterBackButton: noop,
      buttons: [],
      cancelOnStateChange: true
    }, opts || {});

    console.log('xxxxxxxxxx', scope, opts)

    function textForIcon(text) {
      if (text && /icon/.test(text)) {
        scope.$actionSheetHasIcon = true;
      }
    }

    for (var x = 0; x < scope.buttons.length; x++) {
      textForIcon(scope.buttons[x].text);
    }

    // Compile the template
    var element = scope.element = $(SHEET_TPL);
    $compile(element)(scope);




    // Grab the sheet element for animation
    var sheetEl = $(element[0].querySelector('.action-sheet-wrapper'));

    var stateChangeListenDone = scope.cancelOnStateChange ?
      $rootScope.$on('$stateChangeSuccess', function() { scope.cancel(); }) :
      noop;

    // removes the actionSheet from the screen
    scope.removeSheet = function(done) {
      if (scope.removed) return;

      scope.removed = true;
      sheetEl.removeClass('action-sheet-up');
      $timeout(function() {
        // wait to remove this due to a 300ms delay native
        // click which would trigging whatever was underneath this
        $ionicBody.removeClass('action-sheet-open');
      }, 400);
      scope.$deregisterBackButton();
      stateChangeListenDone();

      $animate.removeClass(element, 'active').then(function() {
        scope.$destroy();
        element.remove();
        // scope.cancel.$scope is defined near the bottom
        scope.cancel.$scope = sheetEl = null;
        (done || noop)();
      });
    };

    scope.showSheet = function(done) {
      if (scope.removed) return;

      $ionicBody.append(element)
                .addClass('action-sheet-open');

      $animate.addClass(element, 'active').then(function() {
        if (scope.removed) return;
        (done || noop)();
      });
      $timeout(function() {
        if (scope.removed) return;
        sheetEl.addClass('action-sheet-up');
      }, 20, false);
    };

    // registerBackButtonAction returns a callback to deregister the action
    scope.$deregisterBackButton = $ionicPlatform.registerBackButtonAction(
      function() {
        $timeout(scope.cancel);
      },
      IONIC_BACK_PRIORITY.actionSheet
    );

    // called when the user presses the cancel button
    scope.cancel = function() {
      // after the animation is out, call the cancel callback
      scope.removeSheet(opts.cancel);
    };

    scope.buttonClicked = function(index) {
      // Check if the button click event returned true, which means
      // we can close the action sheet
      if (opts.buttonClicked(index, opts.buttons[index]) === true) {
        scope.removeSheet();
      }
    };

    scope.destructiveButtonClicked = function() {
      // Check if the destructive button click event returned true, which means
      // we can close the action sheet
      if (opts.destructiveButtonClicked() === true) {
        scope.removeSheet();
      }
    };

    scope.showSheet();

    // Expose the scope on $ionicActionSheet's return value for the sake
    // of testing it.
    scope.cancel.$scope = scope;

    return scope.cancel;
  }
}]);

  angular.module('com.chivox.plugins', [
    'com.chivox.plugins.aiengine',
    'com.chivox.plugins.chivoxFileTransfer',
    'com.chivox.plugins.jpush',
    'com.chivox.plugins.toast',
    'com.chivox.plugins.actionSheet',
    'com.chivox.plugins.datepicker'
  ]);

})();